//=======================================================================
//                        DE0-Nano-SoC DVI sample
//
// File:
//    DE0_Nano_SoC_DVI.v
//
// Abstract:
//    DE0-Nano-SoC DVI sample.
//
// Created:
//    4 April 2016
//
//======================================================================
//
// Copyright (c) 2016, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: 0ea78e9c03562749c256719d11d2f5787d14c4a6 $

module DE0_Nano_SoC_DVI(
  //////////// CLOCK //////////
  input         FPGA_CLK1_50,
  input         FPGA_CLK2_50,
  input         FPGA_CLK3_50,

  //////////// KEY //////////
  input  [1:0]  KEY,

  //////////// LED //////////
  output [7:0]  LED,

  //////////// SW //////////
  input  [3:0]  SW,

  //////////// GPIO_0, GPIO connect to GPIO Default //////////
  inout  [35:0] GPIO_0,
  //////////// GPIO_1, GPIO connect to GPIO Default //////////
  inout  [35:0] GPIO_1
);

localparam P_VIDEO_MODE="VGA";
//localparam P_VIDEO_MODE="SVGA";
//localparam P_VIDEO_MODE="XGA";
//localparam P_VIDEO_MODE="SXGA";

localparam P_GPIO="GPIO_0";
//localparam P_GPIO="GPIO_1";
   
reg  [31:0]	Cont;
wire [2:0] w_d_p,w_d_n;
wire w_c_p,w_c_n;

genvar i;
generate if (P_GPIO=="GPIO_0") begin: b_gpio
  //assign GPIO_0 = 36'hzzzzzzzz;
  assign GPIO_1 = 36'hzzzzzzzz;

  for (i=0;i<36;i=i+1) begin : b_gpio_0
    if (i==25) assign GPIO_0[i] = w_d_p[2];
    else
    if (i==21) assign GPIO_0[i] = w_d_p[1];
    else
    if (i==14) assign GPIO_0[i] = w_d_p[0];
    else
    if (i==24) assign GPIO_0[i] = w_d_n[2];
    else
    if (i==20) assign GPIO_0[i] = w_d_n[1];
    else
    if (i==15) assign GPIO_0[i] = w_d_n[0];
    else
    if (i==10) assign GPIO_0[i] = w_c_p;
    else
    if (i==11) assign GPIO_0[i] = w_c_n;
    else assign GPIO_0[i] = 'hz;
  end
end else begin
  assign GPIO_0 = 36'hzzzzzzzz;
  //assign GPIO_1 = 36'hzzzzzzzz;

  for (i=0;i<36;i=i+1) begin : b_gpio_0
    if (i==25) assign GPIO_1[i] = w_d_p[2];
    else
    if (i==21) assign GPIO_1[i] = w_d_p[1];
    else
    if (i==14) assign GPIO_1[i] = w_d_p[0];
    else
    if (i==24) assign GPIO_1[i] = w_d_n[2];
    else
    if (i==20) assign GPIO_1[i] = w_d_n[1];
    else
    if (i==15) assign GPIO_1[i] = w_d_n[0];
    else
    if (i==10) assign GPIO_1[i] = w_c_p;
    else
    if (i==11) assign GPIO_1[i] = w_c_n;
    else assign GPIO_0[i] = 'hz;
  end
end
endgenerate

wire clk_v,clk_vx5;
wire w_pll_locked;

always@(posedge FPGA_CLK1_50 or negedge KEY[0])
  begin
    if(!KEY[0])
      Cont	<=	0;
    else
      Cont	<=	Cont+1;
    end
	 
assign	LED =	KEY[0]? {Cont[25:24],Cont[25:24],Cont[25:24],
                         Cont[25:24],Cont[25:24]}:10'h3ff;
generate if (P_VIDEO_MODE=="VGA") begin
  altera_pll #(
    .fractional_vco_multiplier("false"),
    .reference_clock_frequency("50.0 MHz"),
    .operation_mode("direct"),
    .number_of_clocks(2),
    .output_clock_frequency0("25.000000 MHz"),
    .phase_shift0("0 ps"),
    .duty_cycle0(50),
    .output_clock_frequency1("125.000000 MHz"),
    .phase_shift1("0 ps"),
    .duty_cycle1(50),
    .output_clock_frequency2("0 MHz"),
    .phase_shift2("0 ps"),
    .duty_cycle2(50),
    .output_clock_frequency3("0 MHz"),
    .phase_shift3("0 ps"),
    .duty_cycle3(50),
    .output_clock_frequency4("0 MHz"),
    .phase_shift4("0 ps"),
    .duty_cycle4(50),
    .output_clock_frequency5("0 MHz"),
    .phase_shift5("0 ps"),
    .duty_cycle5(50),
    .output_clock_frequency6("0 MHz"),
    .phase_shift6("0 ps"),
    .duty_cycle6(50),
    .output_clock_frequency7("0 MHz"),
    .phase_shift7("0 ps"),
    .duty_cycle7(50),
    .output_clock_frequency8("0 MHz"),
    .phase_shift8("0 ps"),
    .duty_cycle8(50),
    .output_clock_frequency9("0 MHz"),
    .phase_shift9("0 ps"),
    .duty_cycle9(50),
    .output_clock_frequency10("0 MHz"),
    .phase_shift10("0 ps"),
    .duty_cycle10(50),
    .output_clock_frequency11("0 MHz"),
    .phase_shift11("0 ps"),
    .duty_cycle11(50),
    .output_clock_frequency12("0 MHz"),
    .phase_shift12("0 ps"),
    .duty_cycle12(50),
    .output_clock_frequency13("0 MHz"),
    .phase_shift13("0 ps"),
    .duty_cycle13(50),
    .output_clock_frequency14("0 MHz"),
    .phase_shift14("0 ps"),
    .duty_cycle14(50),
    .output_clock_frequency15("0 MHz"),
    .phase_shift15("0 ps"),
    .duty_cycle15(50),
    .output_clock_frequency16("0 MHz"),
    .phase_shift16("0 ps"),
    .duty_cycle16(50),
    .output_clock_frequency17("0 MHz"),
    .phase_shift17("0 ps"),
    .duty_cycle17(50),
    .pll_type("General"),
    .pll_subtype("General")
  ) altera_pll_i (
    .rst	(1'b0),
    .outclk	({clk_vx5, clk_v}),
    .locked	(w_pll_locked),
    .fboutclk	( ),
    .fbclk	(1'b0),
    .refclk	(FPGA_CLK1_50)
  );
end else if (P_VIDEO_MODE=="SVGA") begin
  altera_pll #(
    .fractional_vco_multiplier("false"),
    .reference_clock_frequency("50.0 MHz"),
    .operation_mode("direct"),
    .number_of_clocks(2),
    .output_clock_frequency0("40.000000 MHz"),
    .phase_shift0("0 ps"),
    .duty_cycle0(50),
    .output_clock_frequency1("200.000000 MHz"),
    .phase_shift1("0 ps"),
    .duty_cycle1(50),
    .output_clock_frequency2("0 MHz"),
    .phase_shift2("0 ps"),
    .duty_cycle2(50),
    .output_clock_frequency3("0 MHz"),
    .phase_shift3("0 ps"),
    .duty_cycle3(50),
    .output_clock_frequency4("0 MHz"),
    .phase_shift4("0 ps"),
    .duty_cycle4(50),
    .output_clock_frequency5("0 MHz"),
    .phase_shift5("0 ps"),
    .duty_cycle5(50),
    .output_clock_frequency6("0 MHz"),
    .phase_shift6("0 ps"),
    .duty_cycle6(50),
    .output_clock_frequency7("0 MHz"),
    .phase_shift7("0 ps"),
    .duty_cycle7(50),
    .output_clock_frequency8("0 MHz"),
    .phase_shift8("0 ps"),
    .duty_cycle8(50),
    .output_clock_frequency9("0 MHz"),
    .phase_shift9("0 ps"),
    .duty_cycle9(50),
    .output_clock_frequency10("0 MHz"),
    .phase_shift10("0 ps"),
    .duty_cycle10(50),
    .output_clock_frequency11("0 MHz"),
    .phase_shift11("0 ps"),
    .duty_cycle11(50),
    .output_clock_frequency12("0 MHz"),
    .phase_shift12("0 ps"),
    .duty_cycle12(50),
    .output_clock_frequency13("0 MHz"),
    .phase_shift13("0 ps"),
    .duty_cycle13(50),
    .output_clock_frequency14("0 MHz"),
    .phase_shift14("0 ps"),
    .duty_cycle14(50),
    .output_clock_frequency15("0 MHz"),
    .phase_shift15("0 ps"),
    .duty_cycle15(50),
    .output_clock_frequency16("0 MHz"),
    .phase_shift16("0 ps"),
    .duty_cycle16(50),
    .output_clock_frequency17("0 MHz"),
    .phase_shift17("0 ps"),
    .duty_cycle17(50),
    .pll_type("General"),
    .pll_subtype("General")
  ) altera_pll_i (
    .rst	(1'b0),
    .outclk	({clk_vx5, clk_v}),
    .locked	(w_pll_locked),
    .fboutclk	( ),
    .fbclk	(1'b0),
    .refclk	(FPGA_CLK1_50)
  );
end else if (P_VIDEO_MODE=="XGA") begin
  altera_pll #(
    .fractional_vco_multiplier("false"),
    .reference_clock_frequency("50.0 MHz"),
    .operation_mode("direct"),
    .number_of_clocks(2),
    .output_clock_frequency0("65.000000 MHz"),
    .phase_shift0("0 ps"),
    .duty_cycle0(50),
    .output_clock_frequency1("325.000000 MHz"),
    .phase_shift1("0 ps"),
    .duty_cycle1(50),
    .output_clock_frequency2("0 MHz"),
    .phase_shift2("0 ps"),
    .duty_cycle2(50),
    .output_clock_frequency3("0 MHz"),
    .phase_shift3("0 ps"),
    .duty_cycle3(50),
    .output_clock_frequency4("0 MHz"),
    .phase_shift4("0 ps"),
    .duty_cycle4(50),
    .output_clock_frequency5("0 MHz"),
    .phase_shift5("0 ps"),
    .duty_cycle5(50),
    .output_clock_frequency6("0 MHz"),
    .phase_shift6("0 ps"),
    .duty_cycle6(50),
    .output_clock_frequency7("0 MHz"),
    .phase_shift7("0 ps"),
    .duty_cycle7(50),
    .output_clock_frequency8("0 MHz"),
    .phase_shift8("0 ps"),
    .duty_cycle8(50),
    .output_clock_frequency9("0 MHz"),
    .phase_shift9("0 ps"),
    .duty_cycle9(50),
    .output_clock_frequency10("0 MHz"),
    .phase_shift10("0 ps"),
    .duty_cycle10(50),
    .output_clock_frequency11("0 MHz"),
    .phase_shift11("0 ps"),
    .duty_cycle11(50),
    .output_clock_frequency12("0 MHz"),
    .phase_shift12("0 ps"),
    .duty_cycle12(50),
    .output_clock_frequency13("0 MHz"),
    .phase_shift13("0 ps"),
    .duty_cycle13(50),
    .output_clock_frequency14("0 MHz"),
    .phase_shift14("0 ps"),
    .duty_cycle14(50),
    .output_clock_frequency15("0 MHz"),
    .phase_shift15("0 ps"),
    .duty_cycle15(50),
    .output_clock_frequency16("0 MHz"),
    .phase_shift16("0 ps"),
    .duty_cycle16(50),
    .output_clock_frequency17("0 MHz"),
    .phase_shift17("0 ps"),
    .duty_cycle17(50),
    .pll_type("General"),
    .pll_subtype("General")
  ) altera_pll_i (
    .rst	(1'b0),
    .outclk	({clk_vx5, clk_v}),
    .locked	(w_pll_locked),
    .fboutclk	( ),
    .fbclk	(1'b0),
    .refclk	(FPGA_CLK1_50)
  );
end else begin
  // SXGA
  altera_pll #(
    .fractional_vco_multiplier("false"),
    .reference_clock_frequency("50.0 MHz"),
    .operation_mode("direct"),
    .number_of_clocks(2),
    .output_clock_frequency0("108.000000 MHz"),
    .phase_shift0("0 ps"),
    .duty_cycle0(50),
    .output_clock_frequency1("540.000000 MHz"),
    .phase_shift1("0 ps"),
    .duty_cycle1(50),
    .output_clock_frequency2("0 MHz"),
    .phase_shift2("0 ps"),
    .duty_cycle2(50),
    .output_clock_frequency3("0 MHz"),
    .phase_shift3("0 ps"),
    .duty_cycle3(50),
    .output_clock_frequency4("0 MHz"),
    .phase_shift4("0 ps"),
    .duty_cycle4(50),
    .output_clock_frequency5("0 MHz"),
    .phase_shift5("0 ps"),
    .duty_cycle5(50),
    .output_clock_frequency6("0 MHz"),
    .phase_shift6("0 ps"),
    .duty_cycle6(50),
    .output_clock_frequency7("0 MHz"),
    .phase_shift7("0 ps"),
    .duty_cycle7(50),
    .output_clock_frequency8("0 MHz"),
    .phase_shift8("0 ps"),
    .duty_cycle8(50),
    .output_clock_frequency9("0 MHz"),
    .phase_shift9("0 ps"),
    .duty_cycle9(50),
    .output_clock_frequency10("0 MHz"),
    .phase_shift10("0 ps"),
    .duty_cycle10(50),
    .output_clock_frequency11("0 MHz"),
    .phase_shift11("0 ps"),
    .duty_cycle11(50),
    .output_clock_frequency12("0 MHz"),
    .phase_shift12("0 ps"),
    .duty_cycle12(50),
    .output_clock_frequency13("0 MHz"),
    .phase_shift13("0 ps"),
    .duty_cycle13(50),
    .output_clock_frequency14("0 MHz"),
    .phase_shift14("0 ps"),
    .duty_cycle14(50),
    .output_clock_frequency15("0 MHz"),
    .phase_shift15("0 ps"),
    .duty_cycle15(50),
    .output_clock_frequency16("0 MHz"),
    .phase_shift16("0 ps"),
    .duty_cycle16(50),
    .output_clock_frequency17("0 MHz"),
    .phase_shift17("0 ps"),
    .duty_cycle17(50),
    .pll_type("General"),
    .pll_subtype("General")
  ) altera_pll_i (
    .rst	(1'b0),
    .outclk	({clk_vx5, clk_v}),
    .locked	(w_pll_locked),
    .fboutclk	( ),
    .fbclk	(1'b0),
    .refclk	(FPGA_CLK1_50)
  );
end
endgenerate
wire [7:0] w_vr;
wire [7:0] w_vg;
wire [7:0] w_vb;
wire w_vsync_x;
wire w_hsync_x;
wire w_de;

// Video generator
fm_hvc #(.P_VIDEO_MODE(P_VIDEO_MODE)) u_hvc (
  .clk_vi(clk_v),
  .rst_x(w_pll_locked),
  // configuration registers
  .i_video_start(1'b1),
  // video out
  .o_r(w_vr),
  .o_g(w_vg),
  .o_b(w_vb),
  .o_vsync_x(w_vsync_x),
  .o_hsync_x(w_hsync_x),
  .o_blank_x(w_de)
);

// DVI encoder
fm_dvi u_dvi (
  .clk_pix(clk_v),
  .clk_pix_x5(clk_vx5),
  .rst_x(w_pll_locked),
  .i_cr(w_vr),
  .i_cg(w_vg),
  .i_cb(w_vb),
  .i_vs(w_vsync_x),
  .i_hs(w_hsync_x),
  .i_de(w_de),
  .o_d_p(w_d_p),
  .o_d_n(w_d_n),
  .o_c_p(w_c_p),
  .o_c_n(w_c_n)
 );

endmodule
