//=======================================================================
//                        DE0-Nano-SoC DVI sample
//
// File:
//   fm_hvc.v
//
// Abstract:
//   HV counter core
//
// Created:
//    4 April 2016
//
//======================================================================
//
// Copyright (c) 2016, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: 1b238f2625ae9f8b2dea7ebf9a268885b05233e3 $

module fm_hvc (
    clk_vi,
    rst_x,
    // configuration registers
    i_video_start,
    // video out
    o_r,
    o_g,
    o_b,
    o_vsync_x,
    o_hsync_x,
    o_blank_x
);
parameter P_VIDEO_MODE="VGA";
//////////////////////////////////
// I/O port definition
//////////////////////////////////
    input          clk_vi;
    input          rst_x;
    // configuration registers
    input          i_video_start;
    // video out timing
    output [7:0]   o_r;
    output [7:0]   o_g;
    output [7:0]   o_b;
    output         o_vsync_x;
    output         o_hsync_x;
    output         o_blank_x;
//////////////////////////////////
// reg
//////////////////////////////////
    reg    [11:0]   r_hcnt;
    reg    [11:0]   r_vcnt;
    reg             r_vsync_x;
    reg             r_hsync_x;
    reg             r_blank_x;

    reg    [7:0]   r_r;
    reg    [7:0]   r_g;
    reg    [7:0]   r_b;

//////////////////////////////////
// wire
//////////////////////////////////
    wire           w_h_end;
    wire           w_v_end;

    wire           w_vsync;
    wire           w_hsync;
    wire           w_hactive;
    wire           w_vactive;
    wire           w_active;

    wire [2:0]     w_test_rgb;
    wire [7:0] w_r_test;
    wire [7:0] w_g_test;
    wire [7:0] w_b_test;

    wire   [2:0]   w_hsel;
    wire           w_vsel;
//////////////////////////////////
// assign
//////////////////////////////////
assign w_r_test = {8{w_test_rgb[2]}};
assign w_g_test = {8{w_test_rgb[1]}};
assign w_b_test = {8{w_test_rgb[0]}};

generate if (P_VIDEO_MODE == "VGA") begin
  assign w_hsel = (r_hcnt >= ((640/8)*7)+'d160) ? 3'd7 :
                  (r_hcnt >= ((640/8)*6)+'d160) ? 3'd6 :
                  (r_hcnt >= ((640/8)*5)+'d160) ? 3'd5 :
                  (r_hcnt >= ((640/8)*4)+'d160) ? 3'd4 :
                  (r_hcnt >= ((640/8)*3)+'d160) ? 3'd3 :
                  (r_hcnt >= ((640/8)*2)+'d160) ? 3'd2 :
                  (r_hcnt >= ((640/8)*1)+'d160) ? 3'd1 : 3'd0;
  assign w_vsel = (r_vcnt >= (480/2)+'d45) ? 1'd1 : 1'b0;
end else if (P_VIDEO_MODE == "SVGA") begin
  assign w_hsel = (r_hcnt >= ((800/8)*7)+'d256) ? 3'd7 :
                  (r_hcnt >= ((800/8)*6)+'d256) ? 3'd6 :
                  (r_hcnt >= ((800/8)*5)+'d256) ? 3'd5 :
                  (r_hcnt >= ((800/8)*4)+'d256) ? 3'd4 :
                  (r_hcnt >= ((800/8)*3)+'d256) ? 3'd3 :
                  (r_hcnt >= ((800/8)*2)+'d256) ? 3'd2 :
                  (r_hcnt >= ((800/8)*1)+'d256) ? 3'd1 : 3'd0;
  assign w_vsel = (r_vcnt >= (600/2)+'d28) ? 1'd1 : 1'b0;
end else if (P_VIDEO_MODE == "XGA") begin
  assign w_hsel = (r_hcnt >= ((1024/8)*7)+'d320) ? 3'd7 :
                  (r_hcnt >= ((1024/8)*6)+'d320) ? 3'd6 :
                  (r_hcnt >= ((1024/8)*5)+'d320) ? 3'd5 :
                  (r_hcnt >= ((1024/8)*4)+'d320) ? 3'd4 :
                  (r_hcnt >= ((1024/8)*3)+'d320) ? 3'd3 :
                  (r_hcnt >= ((1024/8)*2)+'d320) ? 3'd2 :
                  (r_hcnt >= ((1024/8)*1)+'d320) ? 3'd1 : 3'd0;
  assign w_vsel = (r_vcnt >= (768/2)+'d38) ? 1'd1 : 1'b0;
end else begin
  // SXGA
  assign w_hsel = (r_hcnt >= ((1280/8)*7)+'d408) ? 3'd7 :
                  (r_hcnt >= ((1280/8)*6)+'d408) ? 3'd6 :
                  (r_hcnt >= ((1280/8)*5)+'d408) ? 3'd5 :
                  (r_hcnt >= ((1280/8)*4)+'d408) ? 3'd4 :
                  (r_hcnt >= ((1280/8)*3)+'d408) ? 3'd3 :
                  (r_hcnt >= ((1280/8)*2)+'d408) ? 3'd2 :
                  (r_hcnt >= ((1280/8)*1)+'d408) ? 3'd1 : 3'd0;
  assign w_vsel = (r_vcnt >= (1024/2)+'d42) ? 1'd1 : 1'b0;
end
endgenerate

assign w_test_rgb = f_rgb(w_hsel, w_vsel);

generate if (P_VIDEO_MODE == "VGA") begin
    // VGA : 60Hz
    assign w_h_end = (r_hcnt == 'd799);  // 800 clock
    assign w_v_end = w_h_end & (r_vcnt == 'd524);  // 525 line

    assign w_hsync = ((r_hcnt >= 'd16)&(r_hcnt <= 'd111)) ? 1'b0 : 1'b1;
    assign w_vsync = ((r_vcnt == 'd10) | (r_vcnt == 'd11)) ? 1'b0 : 1'b1;

    assign w_hactive = ((r_hcnt >= 'd160)&(r_hcnt <= 'd799)) ? 1'b1 : 1'b0;
    assign w_vactive = ((r_vcnt >= 'd45)&(r_vcnt <= 'd524))  ? 1'b1 : 1'b0;
end else if (P_VIDEO_MODE == "SVGA") begin
    // SVGA : 60Hz
    assign w_h_end = (r_hcnt == 'd1055);  // 1056 clock
    assign w_v_end = w_h_end & (r_vcnt == 'd627);  // 628 line

    assign w_hsync = ((r_hcnt >= 'd88)&(r_hcnt <= 'd215)) ? 1'b1 : 1'b0;
    assign w_vsync = ((r_vcnt >= 'd23) & (r_vcnt <= 'd26)) ? 1'b1 : 1'b0;

    assign w_hactive = ((r_hcnt >= 'd256)&(r_hcnt <= 'd1055)) ? 1'b1 : 1'b0;
    assign w_vactive = ((r_vcnt >= 'd28)&(r_vcnt <= 'd627))  ? 1'b1 : 1'b0;
end else if (P_VIDEO_MODE == "XGA")  begin
    // XGA : 60Hz
    assign w_h_end = (r_hcnt == 'd1343);  // 1344 clock
    assign w_v_end = w_h_end & (r_vcnt == 'd805);  // 806 line

    assign w_hsync = ((r_hcnt >= 'd160)&(r_hcnt <= 'd295)) ? 1'b0 : 1'b1;
    assign w_vsync = ((r_vcnt >= 'd29) & (r_vcnt <= 'd34)) ? 1'b0 : 1'b1;

    assign w_hactive = ((r_hcnt >= 'd320)&(r_hcnt <= 'd1343)) ? 1'b1 : 1'b0;
    assign w_vactive = ((r_vcnt >= 'd38)&(r_vcnt <= 'd805))  ? 1'b1 : 1'b0;
end else begin
    // SXGA : 60Hz
    assign w_h_end = (r_hcnt == 'd1687);  // 1688 clock
    assign w_v_end = w_h_end & (r_vcnt == 'd1065);  // 1066 line

    assign w_hsync = ((r_hcnt >= 'd48)&(r_hcnt <= 'd159)) ? 1'b1 : 1'b0;
    assign w_vsync = ((r_vcnt >= 'd1) & (r_vcnt <= 'd3)) ? 1'b1 : 1'b0;

    assign w_hactive = ((r_hcnt >= 'd408)&(r_hcnt <= 'd1687)) ? 1'b1 : 1'b0;
    assign w_vactive = ((r_vcnt >= 'd42)&(r_vcnt <= 'd1065))  ? 1'b1 : 1'b0;
end
endgenerate
   
    assign w_active = w_hactive & w_vactive;

    // color should be black in blanking
    //assign w_r = (w_active) ? w_rgb[7:0]   : 8'h00;
    //assign w_g = (w_active) ? w_rgb[15:8]  : 8'h00;
    //assign w_b = (w_active) ? w_rgb[23:16] : 8'h00;

    assign o_vsync_x = r_vsync_x;
    assign o_hsync_x = r_hsync_x;
    assign o_blank_x = r_blank_x;

    assign o_r = r_r;
    assign o_g = r_g;
    assign o_b = r_b;

function [2:0]  f_rgb;
  input [2:0] hsel;
  input vsel;
  reg [2:0] col;
  begin
    col = 3'b000;
    case (hsel)
      3'd0: col = 3'b111;  // white 
      3'd1: col = 3'b110;
      3'd2: col = 3'b011;
      3'd3: col = 3'b010;
      3'd4: col = 3'b101;
      3'd5: col = 3'b100;
      3'd6: col = 3'b001;
      3'd7: col = 3'b000;  // black
    endcase
    f_rgb = (vsel) ? ~col : col;
  end
endfunction


//////////////////////////////////
// always
//////////////////////////////////
    // H counter
    always @(posedge clk_vi or negedge rst_x) begin
        if (~rst_x) begin
            r_hcnt <= 'b0;
        end else begin
            if (w_h_end) r_hcnt <= 'b0;
            else r_hcnt <= r_hcnt + 1'b1;
        end
    end

    // V counter
    always @(posedge clk_vi or negedge rst_x) begin
        if (~rst_x) begin
            r_vcnt <= 'd0;
        end else begin
            if (w_v_end) r_vcnt <= 'd0;
            else if (w_h_end) r_vcnt <= r_vcnt + 1'b1;
        end
    end

   // sync
    always @(posedge clk_vi or negedge rst_x) begin
        if (~rst_x) begin
            r_vsync_x <= 1'b1;
            r_hsync_x <= 1'b1;
            r_blank_x <= 1'b1;
        end else begin
            r_vsync_x <= w_vsync;
            r_hsync_x <= w_hsync;
            r_blank_x <= w_active;
        end
    end


    always @(posedge clk_vi) begin
      r_r <= (w_active) ? w_r_test : 8'h00;
      r_g <= (w_active) ? w_g_test : 8'h00;
      r_b <= (w_active) ? w_b_test : 8'h00;
    end
endmodule
