//=======================================================================
//                        DE0-Nano-SoC DVI sample
//
// File:
//   fm_serialize.v
//
// Abstract:
//   DVI serializer
//  Created:
//    11 March 2014
//======================================================================
//
// Copyright (c) 2014, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// $Id: ffb79abcd7634cc0f048be5a2de54ae6a7b6fe9a $

module fm_serialize (
  clk_pix,
  clk_pix_x5,
  rst_x,
  i_d,
  o_s_l,
  o_s_h
);

////////////////////////////
// I/O definition
////////////////////////////
input         clk_pix;
input         clk_pix_x5;
input         rst_x;
input  [9:0]  i_d;
output        o_s_l;
output        o_s_h;
/////////////////////////
//  Register definition
/////////////////////////
reg  [4:0]  r_e;
reg  [4:0]  r_o;
reg         r_s_l;
reg         r_s_h;
reg         r_toggle;
reg         r_toggle_1z;
   
/////////////////////////
//  wire definition
/////////////////////////
wire   w_load_new_data;
// //////////////////////
//  assign statement
/////////////////////////
assign w_load_new_data = r_toggle ^ r_toggle_1z;

assign o_s_l = r_s_l;
assign o_s_h = r_s_h;
////////////////////////
// always 
///////////////////////

  always @(posedge clk_pix or negedge rst_x) begin
    if (~rst_x) begin
      r_toggle <= 1'b0;
    end else begin
      r_toggle <= ~r_toggle;
    end
  end
   
// x5       0 1 2 3 4 5 6 7 8 9 10
// load    1         1         1
// odd out  1 3 5 7 9 1 3 5 7 9 1
// evv out  0 2 4 6 8 0 2 4 6 8 0

// edge detect register
  always @(posedge clk_pix_x5) begin
    r_toggle_1z <= r_toggle;
  end


  always @(posedge clk_pix_x5) begin
    if (w_load_new_data) begin
      r_o <= {i_d[9],i_d[7],i_d[5],i_d[3],i_d[1]};
      r_e <= {i_d[8],i_d[6],i_d[4],i_d[2],i_d[0]};
    end else begin
      r_o <= {1'b0,r_o[4:1]};
      r_e <= {1'b0,r_e[4:1]};
    end
  end

  always @(posedge clk_pix_x5) begin
    r_s_l <= r_o[0];
    r_s_h <= r_e[0];
  end

endmodule


